@@ -20,6 +20,7 @@
 
 #include <config.h>
 #include <stdio.h>
+#include <string.h>
 #include <stdlib.h>
 #include <assert.h>
 
@@ -244,14 +245,21 @@ mpi_print( FILE *fp, MPI a, int mode )
 u32
 mpi_get_keyid( MPI a, u32 *keyid )
 {
-#if BYTES_PER_MPI_LIMB != 4
-  #error Make this function work with other LIMB sizes
-#endif
+#if BYTES_PER_MPI_LIMB == 4
     if( keyid ) {
 	keyid[0] = a->nlimbs >= 2? a->d[1] : 0;
 	keyid[1] = a->nlimbs >= 1? a->d[0] : 0;
     }
     return a->nlimbs >= 1? a->d[0] : 0;
+#elif BYTES_PER_MPI_LIMB == 8
+    if( keyid ) {
+	keyid[0] = a->nlimbs? (u32)(a->d[0] >> 32) : 0;
+	keyid[1] = a->nlimbs? (u32)(a->d[0] & 0xffffffff) : 0;
+    }
+    return a->nlimbs? (u32)(a->d[0] & 0xffffffff) : 0;
+#else
+  #error Make this function work with other LIMB sizes
+#endif
 }
 
 
@@ -319,6 +327,15 @@ mpi_set_buffer( MPI a, const byte *buffer, unsigned nbytes, int sign )
 	alimb |= *p-- << 16 ;
 	alimb |= *p-- << 24 ;
       #elif BYTES_PER_MPI_LIMB == 8
+	/* cast due to egc's "left shift count >= width of type" warning*/
+	alimb  = (mpi_limb_t)*p--	;
+	alimb |= (mpi_limb_t)*p-- <<  8 ;
+	alimb |= (mpi_limb_t)*p-- << 16 ;
+	alimb |= (mpi_limb_t)*p-- << 24 ;
+	alimb |= (mpi_limb_t)*p-- << 32 ;
+	alimb |= (mpi_limb_t)*p-- << 40 ;
+	alimb |= (mpi_limb_t)*p-- << 48 ;
+	alimb |= (mpi_limb_t)*p-- << 56 ;
       #else
 	#error please implement for this limb size.
       #endif
@@ -331,14 +348,14 @@ mpi_set_buffer( MPI a, const byte *buffer, unsigned nbytes, int sign )
 	if( p >= buffer ) alimb |= *p-- << 16 ;
 	if( p >= buffer ) alimb |= *p-- << 24 ;
       #elif BYTES_PER_MPI_LIMB == 8
-	alimb  = *p--	    ;
-	if( p >= buffer ) alimb |= *p-- <<  8 ;
-	if( p >= buffer ) alimb |= *p-- << 16 ;
-	if( p >= buffer ) alimb |= *p-- << 24 ;
-	if( p >= buffer ) alimb |= *p-- << 32 ;
-	if( p >= buffer ) alimb |= *p-- << 40 ;
-	if( p >= buffer ) alimb |= *p-- << 48 ;
-	if( p >= buffer ) alimb |= *p-- << 56 ;
+	alimb  = (mpi_limb_t)*p-- ;
+	if( p >= buffer ) alimb |= (mpi_limb_t)*p-- <<	8 ;
+	if( p >= buffer ) alimb |= (mpi_limb_t)*p-- << 16 ;
+	if( p >= buffer ) alimb |= (mpi_limb_t)*p-- << 24 ;
+	if( p >= buffer ) alimb |= (mpi_limb_t)*p-- << 32 ;
+	if( p >= buffer ) alimb |= (mpi_limb_t)*p-- << 40 ;
+	if( p >= buffer ) alimb |= (mpi_limb_t)*p-- << 48 ;
+	if( p >= buffer ) alimb |= (mpi_limb_t)*p-- << 56 ;
       #else
 	#error please implement for this limb size.
       #endif
